#!/bin/bash

# -------------------------------------------------------------------------------
# Filename:    check_newfiles.sh
# Revision:    1.0
# Date:        2007/10/01
# Author:      Luigi Balzano
# Email:       info@sistemistica.it
# Website:     www.sistemistica.it
# Description: Plugin that check existence of new files since last execution in a
#              specified folder.
# Notes:       Useful to check error files (e.g. rejected log files) in a folder
# -------------------------------------------------------------------------------
# Copyright:   2007 (c) Luigi Balzano
# License:     GPL
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# you should have received a copy of the GNU General Public License
# along with this program (or with Nagios); if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA
# -------------------------------------------------------------------------------
# Credits go to Ethan Galstad for coding Nagios
# If any changes are made to this script, please mail me a copy of the changes
#

PROGNAME=`basename $0`
PROGPATH=`echo $0 | /bin/sed -e 's,[\\/][^\\/][^\\/]*$,,'`
REVISION=`echo '$Revision: 1.0 $' | sed -e 's/[^0-9.]//g'`
TIMESTAMP=`date +%Y%m%d%H%M%S`

. $PROGPATH/utils.sh

print_usage() {
        echo "Usage: $PROGNAME -h"
        echo "Usage: $PROGNAME -v"
        echo "Usage: $PROGNAME -f CONFIGFILE"
        echo ""
        echo "Path of CONFIGFILE must be relative to the program path"
        echo "(e.g. no path needed if config file is in the program dir)"
}

print_help() {
        print_revision $PROGNAME $REVISION
        echo ""
        print_usage
        echo ""
        echo "Plugin that check existence of new files since last execution in a"
        echo "specified folder."
        echo ""
        support
}

# No command line arguments are supplied for this script. Print help

if [ "$#" -eq "0" ]; then
        print_usage
        exit $STATE_UNKNOWN
fi

# Print help with --help

if [[ ("$#" -eq "1") && ("$1" = "--help") ]]; then
        print_help
        exit $STATE_UNKNOWN
fi

unset CONFIGFILE

while getopts "vhf:" Option
do
  case $Option in
    h     ) print_help
            exit $STATE_UNKNOWN;;
    v     ) print_revision $PROGNAME $REVISION
            exit $STATE_UNKNOWN;;
    f     ) CONFIGFILE=$OPTARG;;
    *     ) print_usage
            exit $STATE_UNKNOWN;;   # DEFAULT
  esac
done

if [ ! -n "$CONFIGFILE" ]
  then
        echo "Unspecified argument"
        print_usage
        exit $STATE_UNKNOWN
fi

if [ -f $PROGPATH/$CONFIGFILE ]
  then
        . "$PROGPATH/$CONFIGFILE"
  else
        echo "Config file not found"
        print_usage
        exit $STATE_UNKNOWN
fi

# Read last status in format LASTMODTIME;LASTFILE1;LASTFILE2;..;LASTFILEn
# Where LASTMODTIME is the modification time of the newest file
# and LASTFILEx is the name of the file with this modification time
# Note: There could be more than one file with the same modification time

STATUS=""
if [ -f $PROGPATH/$STATUSFILE ]
  then
        . "$PROGPATH/$STATUSFILE"
fi

if [ ! -d "$CHECKDIR" ]
  then
    echo "WARNING - Directory specified doesn't exists or isn't valid: $CHECKDIR"
    exit $STATE_WARNING
fi

# Print the first modification time greater or equal last status, and the name of all new files from that time
# File name list is sort by time (last name is the newest file and should be next "last status")

FILELIST=`find "$CHECKDIR" -type f -maxdepth 1 -printf "%T@;%f\n" 2>&- | sort -t ";" -k 1 -g | awk -F ";" -v last="$STATUS" '
BEGIN {
  split(last,parts,";");
  checklasttime=parts[1];
  gsub(/[0-9]+/,"",checklasttime);
  if (length(checklasttime)==0) {
    lasttime=parts[1];
    delete parts[1];
    totfiles=0;
    for (filename in parts) totfiles++;
  } else {
    lasttime=0;
    delete parts;
    totfiles=0;
  }
  strout="";
}
{
  if ($1 > lasttime) {
    if (length(strout)==0) strout=$1;
    strout=strout";"$2;
  } else {
    if (($1==lasttime) && (totfiles>0)) {
      found=0;
      for (filename in parts) { if (parts[filename]==$2) { found=1; } }
      if (found==0) {
        if (length(strout)==0) strout=$1;
        strout=strout";"$2;
      }
    }
  }
}
END { print strout; }'`

# echo "#${FILELIST}#"

if [ -z "$FILELIST" ]
  then
    # No new or modified file found
    if [ -z "$STATUS" ]
      then
        # No last status available
        LASTMODTIME="unavailable"
      else
        # Print last status available
        STATUSTIMEPART=`echo "$STATUS" | awk -F ";" '{ print $1 }'`
        LASTMODTIME=`date +"%Y-%m-%d %H:%M:%S" --date="00:00:00 1970-01-01 UTC + $STATUSTIMEPART seconds"`
    fi
    echo "OK - No file was added or modified in folder $CHECKDIR since last check (last modification time is $LASTMODTIME)"
    exit $STATE_OK
  else
    # Save new current status
    LASTFILEFOUND=`echo "$FILELIST" | awk -F ";" '{ print $NF }'`
    # echo "LASTFILEFOUND: $LASTFILEFOUND"
    NEWSTATUS=`find "$CHECKDIR"/"$LASTFILEFOUND" -type f -maxdepth 1 -printf "%T@;%f\n" 2>&-`
    # echo "NEW STATUS: $NEWSTATUS"
    echo "STATUS=\"${NEWSTATUS}\"" > "$PROGPATH/$STATUSFILE"

    # Build output message
    if [ -z "$STATUS" ]
      then
        # No last status available
        OLDLASTMODTIME="unavailable"
      else
        # Start from last status available
        STATUSTIMEPART=`echo "$STATUS" | awk -F ";" '{ print $1 }'`
        OLDLASTMODTIME=`date +"%Y-%m-%d %H:%M:%S" --date="00:00:00 1970-01-01 UTC + $STATUSTIMEPART seconds"`
    fi

    NEWSTATUSTIMEPART=`echo "$NEWSTATUS" | awk -F ";" '{ print $1 }'`
    NEWLASTMODTIME=`date +"%Y-%m-%d %H:%M:%S" --date="00:00:00 1970-01-01 UTC + $NEWSTATUSTIMEPART seconds"`

    FILELISTTIMEPART=`echo "$FILELIST" | awk -F ";" '{ print $1 }'`
    FILELIST=${FILELIST#"${FILELISTTIMEPART};"}

    echo "CRITICAL - The following file(s) was added or modified in folder $CHECKDIR since last check (last modification time changes from $OLDLASTMODTIME to $NEWLASTMODTIME): $FILELIST"
    exit $STATE_CRITICAL
fi
